

summarize<-function(est.beta, true.beta, est.S, S){
  v1 = norm(as.matrix(est.beta - true.beta))
  v2 = norm(as.matrix(est.beta - true.beta),type="E")^2
  v3 = length(which(est.S%in%S == TRUE))
  v4 = length(which(est.S%in%S == FALSE))
  return(c(v1,v2,v3,v4))
}

gendata <- function(case, randSeed = 0){
  set.seed(0)
  if(case == 1){
    n = 400; p = 5000; rho = 0.5
    non.zeros = 6; z = abs(rnorm(non.zeros)); signs = rbinom(non.zeros,1,0.5)
    zero.entries = which(signs==0); signs[zero.entries] = -1
    f1 = 4*log(n)/sqrt(n); f2 = z/4; b = (f1 + f2)*signs
    true.beta = rep(0,p)
    true.beta[1:non.zeros] = b
    set.seed(randSeed)
    x = matrix(rnorm(n*p, mean=0, sd=1), n, p)
    y = x[, 1:non.zeros]%*%b + rnorm(n)
    
  } else if(case == 2){
    n = 400; p = 5000; rho = 0.5
    non.zeros = 6; z = abs(rnorm(non.zeros)); signs = rbinom(non.zeros,1,0.5)
    zero.entries = which(signs==0); signs[zero.entries] = -1
    f1 = 4*log(n)/sqrt(n); f2 = z/4; b = (f1 + f2)*signs
    true.beta = rep(0,p)
    true.beta[1:non.zeros] = b
    set.seed(randSeed)
    corrmat = diag(rep(1-rho, p)) + matrix(rho, p, p)
    cholmat = chol(corrmat)
    x = matrix(rnorm(n*p, 0, 1), n, p)
    x = x%*%cholmat
    feta = x[, 1:non.zeros]%*%b; fprob = exp(feta)/(1+exp(feta))
    y = rbinom(n, 1, fprob)
  } else if(case == 3){
    n = 400; p = 5000; rho = 0.5
    b = c(0.6,0.6,0.6,-0.9*sqrt(2))
    true.beta = rep(0,p)
    true.beta[1:4] = b
    non.zeros = 4
    set.seed(randSeed)
    corrmat = diag(rep(1-rho, p)) + matrix(rho, p, p)
    corrmat[,4] = sqrt(rho)
    corrmat[4, ] = sqrt(rho)
    corrmat[4,4] = 1
    cholmat = chol(corrmat)
    x = matrix(rnorm(n*p, 0, 1), n, p)
    x = x%*%cholmat
    myrates = exp(x[, 1:4]%*%b)
    y = rpois(n, myrates)
  } else if(case == 4){
    n = 400; p = 5000; rho = 0.5
    non.zeros = 5
    b = c(4,4,4,-6*sqrt(2),4/3)
    true.beta = rep(0,p)
    true.beta[1:5] = b
    set.seed(randSeed)
    corrmat = diag(rep(1-rho, p)) + matrix(rho, p, p)
    corrmat[,4] = sqrt(rho)
    corrmat[4, ] = sqrt(rho)
    corrmat[4,4] = 1
    corrmat[,5] = 0
    corrmat[5, ] = 0
    corrmat[5,5] = 1
    cholmat = chol(corrmat)
    x = matrix(rnorm(n*p, mean=0, sd=1), n, p)
    x = x%*%cholmat
    myrates = exp(x[, 1:5]%*%b)
    Sur = rexp(n,myrates); CT = rexp(n,0.1)
    Z = pmin(Sur,CT); ind = as.numeric(Sur<=CT)
    y = Surv(Z,ind)
    
  }
  
  S = 1:non.zeros 
  data = list(x = x, y = y, true.beta = true.beta, S = S)
  return(data)
}


nscmain<-function(xtrain,ytrain,xtest,ytest){

traindata = list(x=xtrain, y=ytrain)

sc.train = pamr.train(traindata)

sc.cv = pamr.cv(sc.train, traindata)

sc.error = sc.cv$error

n=dim(xtrain)[2]
sc.train.error = min(sc.error)

sc.nonzero = sc.cv$size

sc.lam = sc.cv$threshold

sc.selnum = min(sc.nonzero[sc.error==min(sc.error)])

sc.sellam = max(sc.lam[ sc.nonzero== sc.selnum &  sc.error == min(sc.error)])

sc.testid = pamr.predict(sc.train, xtest, sc.sellam)

sc.test.error = mean(sc.testid!=ytest)

c(sc.train.error=sc.train.error, sc.test.error=sc.test.error,sc.selnum=sc.selnum)
}

irmain<-function(x.train,y.train,x.test,y.test){
  x.train1 = x.train[y.train==1, ]
  x.train0 = x.train[y.train==0, ]
  mu1 = apply(x.train1,2,mean)
  mu0 = apply(x.train0,2,mean)
  sd1 = apply(x.train1,2,sd)
  sd0 = apply(x.train0,2,sd)
  v = (sd1^2+sd0^2)/2
  y1 = apply((t(x.test)-mu1)^2/v,2,sum)
  y0 = apply((t(x.test)-mu0)^2/v,2,sum)
  ir.test.error = mean((y1<y0)!=y.test)
  y1 = apply((t(x.train)-mu1)^2/v,2,sum)
  y0 = apply((t(x.train)-mu0)^2/v,2,sum)
  ir.train.error = mean((y1<y0)!=y.train)
  
  c(ir.train.error=ir.train.error, ir.test.error=ir.test.error,ir.selnum=ncol(x.train))
}



xtab<-function(x, x2=NULL, caption=NULL, align=NULL, digits=NULL, rownames=NULL, colnames=NULL){
  m=nrow(x)
  n=ncol(x)
  if(!is.null(rownames(x))) rownames=rownames(x)
  if(!is.null(colnames(x))) colnames=colnames(x)
  cat('% latex table generated by xtab function\n')
  cat('%',date(),'\n')
  cat('\\begin{table}[ht]\n')
  if(!is.null(caption))
    cat('\\caption{',caption,'}\n',sep='')
  
  cat('\\begin{center}\n')
  if(is.null(align)) {
    align = 'l'
    for(i in 1:n) align = paste(align,'r',sep='')
  }
  if(nchar(align)==1){
    atmp=align
    for(i in 1:n) align = paste(align,atmp,sep='')
  }
  
  if(is.null(digits)) digits = rep(2,n)
  if(length(digits)==1) digits = rep(digits,n+1)
  cat('\\begin{tabular}{', align,'}\n', sep='')
  cat('\\hline\n')
  if(!is.null(colnames)){
    for(j in 1:n)
      cat('&',colnames[j],sep='')
    cat('\\\\\n\\hline\n')
  }
  for(i in 1:m){
    cat(rownames[i])
    for(j in 1:n){
      if(!is.character(x[i,j])){
        cat('&',format(round(x[i,j],digits[j]),nsmall=digits[j]),sep='')
      } 
      else
        cat('&',x[i,j],sep='')
      if(!is.null(x2)){
        if(!is.character(x2[i,j]))
          cat('(',format(round(x2[i,j],digits[j]),nsmall=digits[j]),')',sep='')
        else
          cat('(',x2[i,j],')',sep='')
      }
    }
    cat('\\\\\n')
    
  }
  cat('\\hline\n')
  cat('\\end{tabular}\n')
  cat('\\end{center}\n')
  cat('\\end{table}\n')
}

